/*  
 	BIN2HEX

	Utility to convert a binary file into an Intel Hex or Motorola
	S-Record file. Only 16-bit hex files and a 64K address space are
	supported (no segment records).

	Usage: BIN2HEX fname.ext [Offset]
	Where: .ext = .hex or .s19
          Offset = hex value added to all addresses
          Binary input file is fname.bin      

	Written by Mike Douglas
	
	 Ver     Date             Desc
	----   ----------    ------------------------------------------------ 
	 1.0   01/04/2023		Initial version
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#define FILENAME_LEN 81
#define LINE_LEN 32			/* data bytes per hex output line */

/* display_usage - Display how the program is used and then exit */

void display_usage (void)
{
	fprintf(stderr, "Usage: BIN2HEX fname.ext [Offset]\n\n"
						 "Where: .ext = .hex or .s19\n"
						 "       Offset = hex value added to all addresses\n"
						 "       Binary input file is fname.bin\n");
	exit(1);
}

/* main */

int main (int argc, char *argv[])
{
	char	fileName[FILENAME_LEN];
	unsigned char inBuf[LINE_LEN];
	char	*bufPtr;
	FILE	*inFile, *outFile;
	int	byteCount, dataAddr, checkSum, i;
	int	offset = 0;
	int	sRecords = 0;
	
/* Process command line parameters */

	fprintf(stdout,"\nBIN2HEX v1.0\n\n");
	if ((argc < 2) || (argc > 3))
		display_usage();

	strcpy(fileName, argv[1]);
	if ((bufPtr = strrchr(fileName,'.')) == NULL) 
		display_usage();
	*bufPtr = '\0';

	if (tolower(bufPtr[1]) == 's')				/* .S19 for example */
		sRecords = 1;
	else if (tolower(bufPtr[1]) != 'h')			/* .HEX */
		display_usage();

	if (islower(bufPtr[1]))
		strcat(fileName, ".bin");					/* input file */
	else
		strcat(fileName, ".BIN");					/* input file */

	if (argc == 3)
		sscanf(argv[2], "%4x", &offset);
	dataAddr = offset;

	if ((inFile = fopen(fileName, "rb")) == NULL) {
		fprintf(stderr,"Can't open file %s\n", fileName);
		exit(1);
	}
	if ((outFile = fopen(argv[1], "w")) == NULL) {
		fprintf(stderr,"Can't open file %s\n", argv[1]);
		exit(1);
	}

/* Read the binary file and write the hex or s-record file */

	do {
		byteCount = fread(inBuf, 1, LINE_LEN, inFile);
		if (byteCount > 0) {
			if (sRecords) {
				fprintf(outFile, "S1%02X%04X", byteCount+3, dataAddr);
				checkSum = byteCount + (dataAddr >> 8) + (dataAddr & 0xff) + 4;
			}
			else {
				fprintf(outFile, ":%02X%04X00", byteCount, dataAddr);
				checkSum = byteCount + (dataAddr >> 8) + (dataAddr & 0xff);
			}
			for (i=0; i<byteCount; i++) {
				fprintf(outFile,"%02X", inBuf[i]);
				checkSum += inBuf[i];
			}
			fprintf(outFile, "%02X\n", -checkSum & 0xff);
			dataAddr += byteCount;
		}
	} while (!feof(inFile));

/* Finish output file and display results */

	if (sRecords)
		fprintf(outFile, "S9\n");
	else 
		fprintf(outFile, ":00000001FF\n");

	fprintf(stdout,"Start address  = %04X\n", offset);
	fprintf(stdout,"High address = %04X\n", dataAddr-1);
	fprintf(stdout,"Output file: %s\n", argv[1]);

	fclose(inFile);
	fclose(outFile);
	return 0;
}
